#include "Adafruit_ssd1306syp.h"

Adafruit_ssd1306syp::Adafruit_ssd1306syp(int sda, int scl):
Adafruit_GFX(SSD1306_WIDTH,SSD1306_HEIGHT)
{
	m_sda = sda;
	m_scl = scl;
	m_pFramebuffer = 0;
}

Adafruit_ssd1306syp::~Adafruit_ssd1306syp()
{
	//release the memory:
	if(m_pFramebuffer)
		free(m_pFramebuffer);
}

//initialized the ssd1306 in the setup function
bool Adafruit_ssd1306syp::initialize(int height, int yOffset, int flip, int contrast)
{
	//setup the pin mode
	pinMode(m_sda,OUTPUT);
	pinMode(m_scl,OUTPUT);

	//malloc the framebuffer.
	m_pFramebuffer = (unsigned char*)malloc(SSD1306_FBSIZE);
	if(m_pFramebuffer == 0){
		return false;
	}
	memset(m_pFramebuffer,0,SSD1306_FBSIZE);//clear it.

	//write command to the screen registers.
    
	writeCommand(SSD1306_CMD_DISPLAY_OFF);// Display off
	writeCommand(0x00);                   // Set lower column start address for page addressing mode
	writeCommand(0x10);                   // Set higher column start address for page addressing mode
	writeCommand(0x40);                   // Set display start line
	writeCommand(0xB0);                   // Set page start address for page addressing mode
	writeCommand(0xA0+flip);              // Segment remap
	writeCommand(0xA6);                   // Set normal/inverse display
	writeCommand(0xA8);                   // Set multiplex ratio
	writeCommand(height-1);
	writeCommand(0xC0+8*flip);            // Set COM Output scan direction
	writeCommand(0xD3);                   // Set display offset
	if (yOffset && (yOffset < height))
		writeCommand(64-yOffset);     // due to mapping of 128x32 display this is valid for that version also
	else
		writeCommand(0x00);           // no offset
	writeCommand(0xD5);                   // Set display clock divide ratio
	writeCommand(0x80);                   //   the suggested ratio is 0x80
	writeCommand(0xD9);                   // Set precharge period
	writeCommand(0xF1);                   //   value for precharge period
	writeCommand(0xDA);                   // Set COM Pins Hardware Configuration
	if (height == 32)
		writeCommand(0x02);
	else
		writeCommand(0x12);
	writeCommand(0xDB);                   // Set V_COMH Deselect level
	writeCommand(0x20);                   //   0.77xVcc
	writeCommand(0x8D);                   // Charge Pump setting
	writeCommand(0x14);                   //   Enable charge pump    
	writeCommand(0x81);                   // Set contrast control
	writeCommand(contrast);               //   doesn't seem to do anything..
	writeCommand(SSD1306_CMD_DISPLAY_ON); // Turn on oled panel 

	delay(10);//wait for the screen loaded.
	return true;
}
void Adafruit_ssd1306syp::clear(bool isUpdateHW)
{
	memset(m_pFramebuffer,0,SSD1306_FBSIZE);//clear the back buffer.
	if(isUpdateHW) update();//update the hw immediately
}

void Adafruit_ssd1306syp::writeCommand(unsigned char cmd)
{
	startIIC();
	writeByte(0x78);  //Slave address,SA0=0
	writeByte(0x00);	//write command
	writeByte(cmd);
	stopIIC();
}
void Adafruit_ssd1306syp::writeByte(unsigned char b)
{
	unsigned char i;
	for(i=0;i<8;i++)
	{
		if((b << i) & 0x80){
			digitalWrite(m_sda, HIGH);
		}else{
			digitalWrite(m_sda, LOW);
		}
		digitalWrite(m_scl, HIGH);
		digitalWrite(m_scl, LOW);
		//    IIC_Byte<<=1;
	}
	digitalWrite(m_sda, HIGH);
	digitalWrite(m_scl, HIGH);
	
	digitalWrite(m_scl, LOW);
}
void Adafruit_ssd1306syp::startIIC()
{
	digitalWrite(m_scl, HIGH);
	digitalWrite(m_sda, HIGH);
	digitalWrite(m_sda, LOW);
	digitalWrite(m_scl, LOW);
}
void Adafruit_ssd1306syp::stopIIC()
{
	digitalWrite(m_scl, LOW);
	digitalWrite(m_sda, LOW);
	digitalWrite(m_scl, HIGH);
	digitalWrite(m_sda, HIGH);	
}

void Adafruit_ssd1306syp::drawPixel(int16_t x, int16_t y, uint16_t color)
{
	unsigned char row;
	unsigned char offset;
	unsigned char  preData;//previous data.
	unsigned char val;
	int16_t  index;

	if ((x < 0) || (x >= width()) || (y < 0) || (y >= height()) || ( m_pFramebuffer==0))	return;

	//get the previous data;
	row = y/8;
	offset =y%8;
	index = row*width() + x;
	preData = m_pFramebuffer[index];

	//set pixel;
	val = 1<<offset;
	if(color!=0)
	{//white! set bit.
		m_pFramebuffer[index] = preData | val;
	}else
	{//black! clear bit.
		m_pFramebuffer[index] = preData & (~val);
	}

}

void Adafruit_ssd1306syp::drawBitmap(int16_t lines, int16_t ofs, const uint8_t bitmap[])
// 128/8 = 16
{
  int o = ofs;
	if ((lines >= height()) || ( m_pFramebuffer==0))	return;

	for (int b = 0; b < lines/8; b++)
  {
	  for (int a = 0; a < 128-ofs; a++)
  	  m_pFramebuffer[b*128+a] = pgm_read_byte_near(bitmap+a+o);
	  o += 128;  
	}
}


void Adafruit_ssd1306syp::startDataSequence()
{
	startIIC();
	writeByte(0x78);
	writeByte(0x40);	//write data
	
}
void Adafruit_ssd1306syp::update()
{
#if 1
	unsigned int  i=0;
	unsigned char m,n;
	for(m=0;m<8;m++)
	{
		writeCommand(0xb0+m);	//page0-page1
		writeCommand(0x00);		//low column start address
		writeCommand(0x10);		//high column start address
		
		startDataSequence();
		for(n=0;n<128;n++)
		{
			writeByte(m_pFramebuffer[i++]);
		}
		stopIIC();
	}
#else
	updateRow(0,SSD1306_MAXROW);
#endif
}

void Adafruit_ssd1306syp::updateRow(int rowID)
{
	unsigned char x;
	unsigned int  index;
	if(rowID>=0 && rowID<SSD1306_MAXROW && m_pFramebuffer)
	{//this part is faster than else.
		//set the position
		startIIC();
		writeByte(0x78);  //Slave address,SA0=0
		writeByte(0x00);	//write command

		writeByte(0xb0+rowID);
		writeByte(((x&0xf0)>>4)|0x10);//|0x10
		writeByte((x&0x0f)|0x01);//|0x01

		stopIIC();

		//start painting the buffer.
		startDataSequence();
		for(x=0;x<SSD1306_WIDTH;x++)
		{
			index = rowID*SSD1306_WIDTH+x;
	  		writeByte(m_pFramebuffer[index]);
		}
		stopIIC();
	}
}
void Adafruit_ssd1306syp::updateRow(int startID, int endID)
{
	unsigned char y =0;
	for(y=startID; y<endID; y++)
	{
		updateRow(y);
	}
}
